#include <SoftwareSerial.h>

SoftwareSerial mySerial(2, 3); // RX, TX
//Pin collegato all'output del divisore del voltaggio
const int FP1 = A2; // Pin connected to voltage divider output
const int FP2 = A0;
const int FP3 = A4;
const int FP4 = A5;
const int FP5 = A3;
//Voltaggio misurato per il pin 5V di Ardino
const float VCC = 4.98; // Measured voltage of Ardunio 5V line
//Resistenza misurata di 3.3k del resistore
const float R_DIV = 47500.0; // Measured resistance of 3.3k resistor
int angle_1, angle_2, angle_3, angle_4, angle_5;

//Fai l'upload del codice, dopo prova ad aggiustare i valori 
//per un grado di piega più accurato
// Upload the code, then try to adjust these values to more
// accurately calculate bend degree.
//resistenza quando dritto
const float STRAIGHT_RESISTANCE = 37300.0; // resistance when straight
//resistenza a 90 gradi
const float BEND_RESISTANCE = 90000.0; // resistance at 90 deg
int volume=0;
int chords=0;
int noteLength=0;
// Il valore della nota MIDI che deve essere suonata
int note = 0; //The MIDI note value to be played
//Collegat al VS1053 Reset line
byte resetMIDI = 4; //Tied to VS1053 Reset line
byte ledPin = 13; //MIDI traffic inidicator
int loopCount=0;
int button=0;

int  instrument = 27;

//questo controlla se la nota sta suonando
boolean notes[10] = {false, false, false, false, false, false, false, false, false, false}; // this checks to see if a note is currently playing
//inserisci il valore delle 10 note qui NON MODIFICARE QUESTO CON FUNZIONI
byte noteValue[10] = {57, 60, 62, 64, 65, 0, 0, 0, 0, 0}; // put the values of the 10 notes here DO NOT MODIFY THIS WITH FUNCTIONS
/*Note Havana: 57, 60, 62, 64, 65 
 Altre note: 60, 62, 64, 65, 67
*/
//controlla quali note dovrebbe suonare questo turno
boolean noteBuffer[10] = {false, false, false, false, false, false, false, false, false, false};  // checks which notes should play this round


void setup() {
  Serial.begin(57600);

  //Setup soft serial for MIDI control
  mySerial.begin(31250);

  //Reset the VS1053
  pinMode(resetMIDI, OUTPUT);
  digitalWrite(resetMIDI, LOW);
  delay(100);
  digitalWrite(resetMIDI, HIGH);
  delay(100);
  //0xB0 è un messaggio canale, imposta il volume del canale verso il massimo (127)
  talkMIDI(0xB0, 0x07, 120); //0xB0 is channel message, set channel volume to near max (127)

}

void loop() {
  angle_1 = calculateAngle(FP1);
  angle_2 = calculateAngle(FP2);
  angle_3 = calculateAngle(FP3);
  angle_4 = calculateAngle(FP4);
  angle_5 = calculateAngle(FP5 );
  Serial.println(" A1: "+ String (angle_1) + " A2: " + String(angle_2) + " A3: " + String(angle_3) + " A4: " + String(angle_4) + " A5: " + String(angle_5));
  
  talkMIDI(0xB0, 0, 0x79); //Bank select Melodic //Default bank GM1
  talkMIDI(0xC0, instrument, 0); //Set instrument number. 0xC0 is a 1 data byte command
   
      if(angle_1 > 90 ){
        noteBuffer[0] = true;
      }
      if(angle_2 > 105){
        noteBuffer[1]=true;
      }
      if(angle_3 > 230){
        noteBuffer[2] = true;
      }
      if(angle_4 > 730){
        noteBuffer[3] = true;   
      }
      if(angle_5 > 120){
        noteBuffer[4] = true;  
      }
      playNotes();
      
      
  delay(50);
}

void playMyNote(byte note, int boolValue){
    // check to see if note is playing
    if(!notes[boolValue]){
      //play note
      noteOn(0, note, 126);
      // array reflects note playing
      notes[boolValue] = true;
      }
  }
void stopMyNote(byte note, int boolValue){
    if(notes[boolValue]){
      noteOff(0, note, 126);
      notes[boolValue] = false;
      }
  }

void stopAllOtherNotes(byte note){
  for(int i = 0; i<10; i++){
    if(noteValue[i] != note){
      stopMyNote(noteValue[i], i);
    }
  }
}

void playNotes(){
  // traverse buffer
  for(int i = 0; i<10; i++){
    if(noteBuffer[i]){
      playMyNote(noteValue[i], i);
      }
    else{stopMyNote(noteValue[i], i);}
    }
   // clear buffer
  for(int i = 0; i<10; i++){
      noteBuffer[i] = false;
    }
  }

  
void noteOn(byte channel, byte note, byte attack_velocity) {
  talkMIDI( (0x90 | channel), note, attack_velocity);
}

//Send a MIDI note-off message.  Like releasing a piano key
void noteOff(byte channel, byte note, byte release_velocity) {
  talkMIDI( (0x80 | channel), note, release_velocity);
}

//suona la nota MIDI. Non controlla se cmd è più grande di 127, oppurese quei valori son inferiori di 127
//Plays a MIDI note. Doesn't check to see that cmd is greater than 127, or that data values are less than 127
void talkMIDI(byte cmd, byte data1, byte data2) {
  digitalWrite(ledPin, HIGH);
  mySerial.write(cmd);
  mySerial.write(data1);

  //Some commands only have one data byte. All cmds less than 0xBn have 2 data bytes 
  //(sort of: http://253.ccarh.org/handout/midiprotocol/)
  if( (cmd & 0xF0) <= 0xB0)
    mySerial.write(data2);

  digitalWrite(ledPin, LOW);
  }
int calculateAngle(int pin){
  int flexADC = analogRead(pin);
  float flexV = flexADC * VCC / 1023.0;
  float flexR = R_DIV * (VCC / flexV - 1.0);
  //Serial.println("Resistance: " + String(flexR) + " ohms");

  // Use the calculated resistance to estimate the sensor's bend angle:
  return map(flexR, STRAIGHT_RESISTANCE, BEND_RESISTANCE,0, 90.0);
}

void turnNotesOff(){
  for(int i = 0; i < 200; i++){
    noteOff(0, i, 120);
  }
}


/*
 Piano:
1 Acoustic Grand Piano
2 Bright Acoustic Piano
3 Electric Grand Piano
4 Honky-tonk Piano
5 Electric Piano 1
6 Electric Piano 2
7 Harpsichord
8 Clavinet
Chromatic Percussion:
9 Celesta
10 Glockenspiel
11 Music Box
12 Vibraphone
13 Marimba
14 Xylophone
15 Tubular Bells
16 Dulcimer
Organ:
17 Drawbar Organ
18 Percussive Organ
19 Rock Organ
20 Church Organ
21 Reed Organ
22 Accordion
23 Harmonica
24 Tango Accordion
Guitar:
25 Acoustic Guitar (nylon)
26 Acoustic Guitar (steel)
27 Electric Guitar (jazz)
28 Electric Guitar (clean)
29 Electric Guitar (muted)
30 Overdriven Guitar
31 Distortion Guitar
32 Guitar harmonics
Bass:
33 Acoustic Bass
34 Electric Bass (finger)
35 Electric Bass (pick)
36 Fretless Bass
37 Slap Bass 1
38 Slap Bass 2
39 Synth Bass 1
40 Synth Bass 2
Strings:
41 Violin
42 Viola
43 Cello
44 Contrabass
45 Tremolo Strings
46 Pizzicato Strings
47 Orchestral Harp
48 Timpani
Strings (continued):
49 String Ensemble 1
50 String Ensemble 2
51 Synth Strings 1
52 Synth Strings 2
53 Choir Aahs
54 Voice Oohs
55 Synth Voice
56 Orchestra Hit
Brass:
57 Trumpet
58 Trombone
59 Tuba
60 Muted Trumpet
61 French Horn
62 Brass Section
63 Synth Brass 1
64 Synth Brass 2
Reed:
65 Soprano Sax
66 Alto Sax
67 Tenor Sax
68 Baritone Sax
69 Oboe
70 English Horn
71 Bassoon
72 Clarinet
Pipe:
73 Piccolo
74 Flute
75 Recorder
76 Pan Flute
77 Blown Bottle
78 Shakuhachi
79 Whistle
80 Ocarina
Synth Lead:
81 Lead 1 (square)
82 Lead 2 (sawtooth)
83 Lead 3 (calliope)
84 Lead 4 (chiff)
85 Lead 5 (charang)
86 Lead 6 (voice)
87 Lead 7 (fifths)
88 Lead 8 (bass + lead)
Synth Pad:
89 Pad 1 (new age)
90 Pad 2 (warm)
91 Pad 3 (polysynth)
92 Pad 4 (choir)
93 Pad 5 (bowed)
94 Pad 6 (metallic)
95 Pad 7 (halo)
96 Pad 8 (sweep)
Synth Effects:
97 FX 1 (rain)
98 FX 2 (soundtrack)
99 FX 3 (crystal)
100 FX 4 (atmosphere)
101 FX 5 (brightness)
102 FX 6 (goblins)
103 FX 7 (echoes)
104 FX 8 (sci-fi)
Ethnic:
105 Sitar
106 Banjo
107 Shamisen
108 Koto
109 Kalimba
110 Bag pipe
111 Fiddle
112 Shanai
Percussive:
113 Tinkle Bell
114 Agogo
115 Steel Drums
116 Woodblock
117 Taiko Drum
118 Melodic Tom
119 Synth Drum
Sound effects:
120 Reverse Cymbal
121 Guitar Fret Noise
122 Breath Noise
123 Seashore
124 Bird Tweet
125 Telephone Ring
126 Helicopter
127 Applause
128 Gunshot
 */

